@@ -336,6 +336,18 @@ int kui_destroy ( struct kuictx *kctx );
  */
 std_list kui_get_map_sets ( struct kuictx *kctx );
 
+/**
+ * This will clear all of the map sets from the KUI.
+ * It will not release the memory associated with these sets.
+ *
+ * \param kctx
+ * The KUI context to clear the sets from
+ *
+ * \return 
+ * 0 on success or -1 on error.
+ */
+int kui_clear_map_sets (struct kuictx *kctx);
+
 /**
  * Add's a kui map set to the kui context.
  *
@@ -378,21 +390,6 @@ int kui_cangetkey ( struct kuictx *kctx );
 
 int kui_getkey ( struct kuictx *kctx );
 
-/**
- * This get's the last sequence of characters read in that made up a
- * CGDB_KEY_*
- * Basically, if CGDB_KEY_PPAGE is returned, this will give back the 
- * characters that made that happen. 
- *
- * \param kctx
- * The kui context.
- *
- * \return
- * NULL if there is no relevant data.
- * Otherwise a pointer to the sequence recieved.
- */
-const char *kui_get_raw_data ( struct kuictx *kctx );
-
 /**
  * Tell's the kui context the maximum number of milliseconds that it is allowed
  * to block while waiting to complete a mapping.
@@ -434,11 +431,19 @@ struct kui_manager;
  *
  * \param stdinfd 
  * The descriptor to read from when looking for the next char
+ * 
+ * \param keycode_timeout
+ * The timeout to use while waiting to match a keyboard code.
+ *
+ * \param mapping_timeout
+ * The timeout to use while waiting to match a mapping sequence.
  *
  * @return
  * A new instance on success, or NULL on error. 
  */
-struct kui_manager *kui_manager_create(int stdinfd );
+struct kui_manager *kui_manager_create (int stdinfd, 
+					unsigned int keycode_timeout,
+					unsigned int mapping_timeout);
 
 /**
  * Destroys a kui context
@@ -475,6 +480,18 @@ int kui_manager_destroy ( struct kui_manager *kuim );
  */
 std_list kui_manager_get_map_sets ( struct kui_manager *kuim );
 
+/**
+ * This will clear all of the map sets from the KUI manager.
+ * It will not release the memory associated with these sets.
+ *
+ * \param kuim
+ * The KUI context to clear the sets from
+ *
+ * \return 
+ * 0 on success or -1 on error.
+ */
+int kui_manager_clear_map_sets (struct kui_manager *kuim);
+
 /**
  * Add's a kui map set to the kui context.
  *
@@ -518,31 +535,12 @@ int kui_manager_cangetkey ( struct kui_manager *kuim );
 int kui_manager_getkey ( struct kui_manager *kuim );
 
 /**
- * This get's the last sequence of characters read in that made up a
- * CGDB_KEY_*
- * Basically, if CGDB_KEY_PPAGE is returned, this will give back the 
- * characters that made that happen. 
- *
- * NOTE: If the user mapped abc to F4, then there currently is no
- * raw data. There is only raw data when the actual terminal key is
- * read. This could potentially be a feature enhancement when needed.
- *
- * \param kuim
- * The kui context.
- *
- * \return
- * NULL if there is no relevant data.
- * Otherwise a pointer to the sequence recieved.
- */
-const char *kui_manager_get_raw_data ( struct kui_manager *kuim );
-
-/**
- * Set's the terminal escape seqeunce time out value.
+ * Set's the terminal escape sequence time out value.
  * This is used to tell CGDB how long to block when looking to match terminal
  * escape sequences. For instance, in order to get F11, Maybe it's necessary
  * for the charachters 27(ESC) 80(P) 81(Q) to get sent. So, if the user types 
- * these * within msec each, then CGDB_KEY_F11 get's returned, otherwise the 
- * key's * are returned as typed.
+ * these within msec each, then CGDB_KEY_F11 get's returned, otherwise the 
+ * key's are returned as typed.
  *
  * \param kuim
  * The kui context
@@ -557,6 +555,23 @@ const char *kui_manager_get_raw_data ( struct kui_manager *kuim );
 int kui_manager_set_terminal_escape_sequence_timeout ( 
 		struct kui_manager *kuim, 
 		unsigned int msec );
+
+/**
+ * Set's the timeout that will be used when matching a mapping.
+ *
+ * \param kuim
+ * The kui context
+ *
+ * \param msec
+ * The maximum number of milliseconds to block while waiting to complete a 
+ * mapping sequence.
+ *
+ * \return
+ * 0 on success, or -1 on error.
+ */
+int kui_manager_set_key_mapping_timeout ( 
+		struct kui_manager *kuim, 
+		unsigned int msec );
 /*@}*/
 
 /* }}} */
