@@ -46,7 +46,6 @@
  * <Bslash>			backslash
  * <Bar>			vertical bar
  * <Del>			delete
- * <EOL>			end-of-line
  * <Up>				cursor-up
  * <Down>			cursor-down
  * <Left>			cursor-left
@@ -125,112 +124,6 @@ enum cgdb_key {
 	CGDB_KEY_CTRL_Y,
 	CGDB_KEY_CTRL_Z,
 
-    /* Alt Letters */
-    CGDB_KEY_ALT_A,
-    CGDB_KEY_ALT_B,
-    CGDB_KEY_ALT_C,
-    CGDB_KEY_ALT_D,
-    CGDB_KEY_ALT_E,
-    CGDB_KEY_ALT_F,
-    CGDB_KEY_ALT_G,
-    CGDB_KEY_ALT_H,
-    CGDB_KEY_ALT_I,
-    CGDB_KEY_ALT_J,
-    CGDB_KEY_ALT_K,
-    CGDB_KEY_ALT_L,
-    CGDB_KEY_ALT_M,
-    CGDB_KEY_ALT_N,
-    CGDB_KEY_ALT_O,
-    CGDB_KEY_ALT_P,
-    CGDB_KEY_ALT_Q,
-    CGDB_KEY_ALT_R,
-    CGDB_KEY_ALT_S,
-    CGDB_KEY_ALT_T,
-    CGDB_KEY_ALT_U,
-    CGDB_KEY_ALT_V,
-    CGDB_KEY_ALT_W,
-    CGDB_KEY_ALT_X,
-    CGDB_KEY_ALT_Y,
-    CGDB_KEY_ALT_Z,
-
-    /* Alt Shifted Letters */
-    CGDB_KEY_ALT_SHIFT_A,
-    CGDB_KEY_ALT_SHIFT_B,
-    CGDB_KEY_ALT_SHIFT_C,
-    CGDB_KEY_ALT_SHIFT_D,
-    CGDB_KEY_ALT_SHIFT_E,
-    CGDB_KEY_ALT_SHIFT_F,
-    CGDB_KEY_ALT_SHIFT_G,
-    CGDB_KEY_ALT_SHIFT_H,
-    CGDB_KEY_ALT_SHIFT_I,
-    CGDB_KEY_ALT_SHIFT_J,
-    CGDB_KEY_ALT_SHIFT_K,
-    CGDB_KEY_ALT_SHIFT_L,
-    CGDB_KEY_ALT_SHIFT_M,
-    CGDB_KEY_ALT_SHIFT_N,
-    CGDB_KEY_ALT_SHIFT_O,
-    CGDB_KEY_ALT_SHIFT_P,
-    CGDB_KEY_ALT_SHIFT_Q,
-    CGDB_KEY_ALT_SHIFT_R,
-    CGDB_KEY_ALT_SHIFT_S,
-    CGDB_KEY_ALT_SHIFT_T,
-    CGDB_KEY_ALT_SHIFT_U,
-    CGDB_KEY_ALT_SHIFT_V,
-    CGDB_KEY_ALT_SHIFT_W,
-    CGDB_KEY_ALT_SHIFT_X,
-    CGDB_KEY_ALT_SHIFT_Y,
-    CGDB_KEY_ALT_SHIFT_Z,
-
-    /* Alt Numbers */
-    CGDB_KEY_ALT_1,
-    CGDB_KEY_ALT_2,
-    CGDB_KEY_ALT_3,
-    CGDB_KEY_ALT_4,
-    CGDB_KEY_ALT_5,
-    CGDB_KEY_ALT_6,
-    CGDB_KEY_ALT_7,
-    CGDB_KEY_ALT_8,
-    CGDB_KEY_ALT_9,
-    CGDB_KEY_ALT_0,
-
-    /* Alt Shifted Numbers */
-    CGDB_KEY_ALT_SHIFT_1,
-    CGDB_KEY_ALT_SHIFT_2,
-    CGDB_KEY_ALT_SHIFT_3,
-    CGDB_KEY_ALT_SHIFT_4,
-    CGDB_KEY_ALT_SHIFT_5,
-    CGDB_KEY_ALT_SHIFT_6,
-    CGDB_KEY_ALT_SHIFT_7,
-    CGDB_KEY_ALT_SHIFT_8,
-    CGDB_KEY_ALT_SHIFT_9,
-    CGDB_KEY_ALT_SHIFT_0,
-
-    /* Alt Special */
-    CGDB_KEY_ALT_MINUS,
-    CGDB_KEY_ALT_EQUAL,
-    CGDB_KEY_ALT_LEFT_BRACKET,
-    CGDB_KEY_ALT_RIGHT_BRACKET,
-    CGDB_KEY_ALT_BACKSLASH,
-    CGDB_KEY_ALT_SEMICOLON,
-    CGDB_KEY_ALT_APOSTROPHE,
-    CGDB_KEY_ALT_COMMA,
-    CGDB_KEY_ALT_PERIOD,
-    CGDB_KEY_ALT_DIVIDE,
-    CGDB_KEY_ALT_ACCENT_MARK,
-
-    /* Alt Shift Special */
-    CGDB_KEY_ALT_SHIFT_UNDERSCORE,
-    CGDB_KEY_ALT_SHIFT_PLUS,
-    CGDB_KEY_ALT_SHIFT_LEFT_CURLY_BRACKET,
-    CGDB_KEY_ALT_SHIFT_RIGHT_CURLY_BRACKET,
-    CGDB_KEY_ALT_SHIFT_PIPE,
-    CGDB_KEY_ALT_SHIFT_COLON,
-    CGDB_KEY_ALT_SHIFT_QUOTE,
-    CGDB_KEY_ALT_SHIFT_LESS_THAN,
-    CGDB_KEY_ALT_SHIFT_GREATER_THAN,
-    CGDB_KEY_ALT_SHIFT_QUESTION_MARK,
-    CGDB_KEY_ALT_SHIFT_TILDA,
-     
     CGDB_KEY_ERROR
 };
 
@@ -260,17 +153,29 @@ struct kui_map_set *kui_term_get_terminal_mappings ( void );
  */
 int kui_term_is_cgdb_key ( int key );
 
+/**
+ * This get's the ascii character representation of the key.
+ *
+ * \param key
+ * The cgdb_key to get the ascii character sequence of.
+ *
+ * \return
+ * The ascii charachter sequence on success. If key is not a cgdb_key
+ * or if this function fails, NULL will return.
+ */
+char *kui_term_get_ascii_char_sequence_from_key (int key);
+
 /**
  * Get's the cgdb_key from the keycode.
  *
  * \param keycode
- * The code to get the corresponding cgdb_key from
+ * The code to get the corresponding key from
  *
  * @return
  * The new key, or -1 on error.
- * CGDB_KEY_ERROR is returned if no cgdb_key matches the keycode.
+ * CGDB_KEY_ERROR is returned if no key matches the keycode.
  */
-int kui_term_get_cgdb_key_from_keycode ( const char *keycode );
+int kui_term_get_key_from_keycode (const char *keycode);
 
 /**
  * Get's the key in string form.
@@ -282,7 +187,7 @@ int kui_term_get_cgdb_key_from_keycode ( const char *keycode );
  * The string form. or NULL on error.
  * if CGDB_KEY_ERROR is passed in, "CGDB_KEY_ERROR" will be returned.
  */
-const char *kui_term_get_string_from_cgdb_key ( int key );
+const char *kui_term_get_string_from_key (int key);
 
 /**
  * Get's the keycode associated with this key.
@@ -323,20 +228,18 @@ const char *kui_term_get_keycode_from_cgdb_key ( int key );
  * @return
  * 0 on success, or -1 on error.
  */
-int kui_term_string_to_cgdb_key_array (
-		const char *string,
-		int **cgdb_key_array );
+int kui_term_string_to_key_array (const char *string, int **key_array);
 
 /**
- * Print's a cgdb_key_array in human readable form. This is mostly used for
+ * Print's a key_array in human readable form. This is mostly used for
  * debugging purposes.
  *
- * \param cgdb_key_array
+ * \param key_array
  * The key array to print.
  *
  * @return
  * 0 on success, or -1 on error.
  */
-int kui_term_print_cgdb_key_array ( int *cgdb_key_array );
+int kui_term_print_key_array (int *key_array);
 
 #endif /* __INPUT_H__ */
