@@ -91,12 +91,12 @@ struct kui_map *kui_map_create (
 	map->original_key = key;
 	map->original_value = value;
 
-	if ( kui_term_string_to_cgdb_key_array ( map->original_key, &map->literal_key ) == -1 ) {
+	if ( kui_term_string_to_key_array ( map->original_key, &map->literal_key ) == -1 ) {
 		kui_map_destroy ( map );
 		return NULL;
 	}
 
-	if ( kui_term_string_to_cgdb_key_array ( map->original_value, &map->literal_value ) == -1 ){
+	if ( kui_term_string_to_key_array ( map->original_value, &map->literal_value ) == -1 ){
 		kui_map_destroy ( map );
 		return NULL;
 	}
@@ -179,7 +179,7 @@ int kui_map_print_cgdb_key_array ( struct kui_map *map ) {
 	if ( !map )
 		return -1;
 
-	if ( kui_term_print_cgdb_key_array ( map->literal_value ) == -1 )
+	if ( kui_term_print_key_array ( map->literal_value ) == -1 )
 		return -1;
 
 	return 0;
@@ -324,14 +324,17 @@ int kui_ms_deregister_map (
 	if ( std_list_get_data ( iter, &data ) == -1 )
 		return -1;
 
-	map = (struct kui_map*)data;
+        /* If the mapping exists, remove it. */
+        if (data) {
+          map = (struct kui_map*)data;
 
-	/* Delete from the tree */
-	if ( kui_tree_delete ( kui_ms->ktree, map->literal_key ) == -1 )
-		return -1;
+          /* Delete from the tree */
+          if ( kui_tree_delete ( kui_ms->ktree, map->literal_key ) == -1 )
+                  return -1;
 
-	if ( std_list_remove ( kui_ms->maps, iter ) == NULL )
-		return -1;
+          if ( std_list_remove ( kui_ms->maps, iter ) == NULL )
+                  return -1;
+        }
 
 	return 0;
 }
@@ -378,14 +381,6 @@ struct kuictx {
 	 * The file descriptor to read from.
 	 */
 	int fd;
-
-	/**
-	 * The literal value of the last map completed.
-	 * This is so that when the client recieves a CGDB_KEY_PPAGE, they can map
-	 * it back to the ascii char's that made up the key, which would be 
-	 * something like "\027\80\81"
-	 */
-	char *last_maps_raw_data;
 };
 
 static int kui_ms_destroy_int_callback ( void *param ) {
@@ -416,7 +411,6 @@ struct kuictx *kui_create(
 	kctx->state_data = state_data;
 	kctx->kui_map_set_list = std_list_create ( NULL );
 	kctx->ms = ms;
-	kctx->last_maps_raw_data = NULL;
 
 	if ( !kctx->kui_map_set_list ) {
 		kui_destroy ( kctx );
@@ -479,6 +473,15 @@ std_list kui_get_map_sets ( struct kuictx *kctx ) {
 	return kctx->kui_map_set_list;
 }
 
+int 
+kui_clear_map_sets (struct kuictx *kctx)
+{
+  if (!kctx)
+    return -1;
+
+  return std_list_remove_all (kctx->kui_map_set_list);
+}
+
 int kui_add_map_set ( 
 		struct kuictx *kctx, 
 		struct kui_map_set *kui_ms ) {
@@ -878,32 +881,6 @@ static int kui_update_buffer (
 		}
 	}
 
-	/* This block is only here to set kctx->last_maps_raw_data.
-	 * Later, a client of the KUI, may want to know what was actually typed to 
-	 * get CGDB_KEY_*
-	 * If the map was found, save the literal_key. Otherwise, set it to NULL. 
-	 */
-	if ( map_was_found ) {
-		int length;
-
-		if ( kctx->last_maps_raw_data ) {
-			free ( kctx->last_maps_raw_data );
-			kctx->last_maps_raw_data = NULL;
-		}
-
-		length = intlen ( the_map_found->literal_key );
-
-		kctx->last_maps_raw_data = (char*)malloc ( sizeof(char)*(length+1) );
-
-		if ( !kctx->last_maps_raw_data )
-			return -1;
-
-		/* Copy null terminated char also */
-		for ( i = 0; i <= length; ++i )
-			kctx->last_maps_raw_data[i] = the_map_found->literal_key[i];
-
-	} 
-
 	return 0;
 }
 
@@ -1012,12 +989,6 @@ int kui_getkey ( struct kuictx *kctx ) {
 	int map_found; 
 	int key;
 
-	/* Set the last_maps raw data value to NULL */
-	if ( kctx->last_maps_raw_data ) {
-		free ( kctx->last_maps_raw_data );
-		kctx->last_maps_raw_data = NULL;
-	}
-
 	/* If a map was found, restart the algorithm. */
 	do {
 		key = kui_findkey ( kctx, &map_found );
@@ -1045,13 +1016,6 @@ int kui_cangetkey ( struct kuictx *kctx ) {
 	return 0;
 }
 
-const char *kui_get_raw_data ( struct kuictx *kctx ) {
-	if ( !kctx )
-		return NULL;
-
-	return kctx->last_maps_raw_data;
-}
-
 int kui_set_blocking_ms ( struct kuictx *kctx, unsigned long msec ) {
 	if ( !kctx )
 		return -1;
@@ -1145,7 +1109,9 @@ int kui_callback (
 	return 1;
 }
 
-struct kui_manager *kui_manager_create(int stdinfd ) {
+struct kui_manager *
+kui_manager_create (int stdinfd, unsigned int keycode_timeout, unsigned int mapping_timeout)
+{
 	struct kui_manager *man;
 
 	man = ( struct kui_manager* )malloc ( sizeof ( struct kui_manager ) );
@@ -1154,7 +1120,7 @@ struct kui_manager *kui_manager_create(int stdinfd ) {
 		return NULL;
 
 	man->normal_keys = NULL;
-	man->terminal_keys = kui_create ( stdinfd, char_callback, 40, NULL );
+	man->terminal_keys = kui_create ( stdinfd, char_callback, keycode_timeout, NULL );
 
 	if ( !man->terminal_keys ) {
 		kui_manager_destroy ( man );
@@ -1166,7 +1132,7 @@ struct kui_manager *kui_manager_create(int stdinfd ) {
 		return NULL;
 	}
 
-	man->normal_keys = kui_create ( -1, kui_callback, 1000, man->terminal_keys );
+	man->normal_keys = kui_create ( -1, kui_callback, mapping_timeout, man->terminal_keys );
 
 	if ( !man->normal_keys ) {
 		kui_manager_destroy ( man );
@@ -1205,6 +1171,15 @@ std_list kui_manager_get_map_sets ( struct kui_manager *kuim ) {
 	return kui_get_map_sets ( kuim->normal_keys );
 }
 
+int 
+kui_manager_clear_map_sets (struct kui_manager *kuim)
+{
+  if (!kuim)
+    return -1;
+
+  return kui_clear_map_sets (kuim->normal_keys);
+}
+
 int kui_manager_add_map_set ( 
 		struct kui_manager *kuim, 
 		struct kui_map_set *kui_ms ) {
@@ -1215,12 +1190,23 @@ int kui_manager_add_map_set (
 	return kui_add_map_set ( kuim->normal_keys, kui_ms );
 }
 
-int kui_manager_cangetkey ( struct kui_manager *kuim ) {
+int 
+kui_manager_cangetkey (struct kui_manager *kuim)
+{
+  if (!kuim)
+    return -1;
 
-	if ( !kuim )
-		return -1;
-
-	return kui_cangetkey ( kuim->normal_keys );
+  /* I'm not sure if checking the terminal keys here is the best solution.
+   *
+   * It might make more sense to flow the extra characters read from during
+   * the terminal keys processing up to the normal keys buffer after a 
+   * mapping is found.
+   *
+   * For now this seems to work. Essentially, the next read get's the buffered 
+   * terminal keys first which is what should happen anyways.
+   */
+  return kui_cangetkey (kuim->terminal_keys) || 
+	 kui_cangetkey (kuim->normal_keys);
 }
 
 int kui_manager_getkey ( struct kui_manager *kuim ) {
@@ -1231,12 +1217,6 @@ int kui_manager_getkey ( struct kui_manager *kuim ) {
 
 }
 
-const char *kui_manager_get_raw_data ( struct kui_manager *kuim ) {
-	if (!kuim)
-		return NULL;
-	return kui_get_raw_data ( kuim->terminal_keys );
-}
-
 int kui_manager_set_terminal_escape_sequence_timeout ( 
 		struct kui_manager *kuim, 
 		unsigned int msec ) {
@@ -1246,4 +1226,13 @@ int kui_manager_set_terminal_escape_sequence_timeout (
 	return kui_set_blocking_ms ( kuim->terminal_keys, msec );
 }
 
+int 
+kui_manager_set_key_mapping_timeout (struct kui_manager *kuim, unsigned int msec)
+{
+  if (!kuim)
+    return -1;
+
+  return kui_set_blocking_ms (kuim->normal_keys, msec);
+}
+
 /* }}} */
