@@ -107,6 +107,9 @@ static char *debugger_path = NULL;	/* Path to debugger to use */
 
 struct kui_manager *kui_ctx = NULL;	/* The key input package */
 
+struct kui_map_set *kui_map = NULL;
+struct kui_map_set *kui_imap = NULL;
+
 int resize_pipe[2] = { -1, -1 };
 
 /* Readline interface */
@@ -297,7 +300,10 @@ rlctx_send_user_command (char *line)
 
   /* This happens when rl_callback_read_char gets EOF */
   if (line == NULL)
-    return;
+  {
+    cleanup ();
+    exit (0);
+  }
 
   /* Add the line passed in to the current line */
   ibuf_add (current_line, line);
@@ -402,11 +408,11 @@ tab_completion_create (char **matches, int num_matches, int max_length)
   int i;
   tab_completion_ptr comptr;
 
-  comptr = (tab_completion_ptr) xmalloc (sizeof (struct tab_completion_ctx));
+  comptr = (tab_completion_ptr) cgdb_malloc (sizeof (struct tab_completion_ctx));
 
-  comptr->matches = xmalloc (sizeof (char *) * (num_matches + 1));
+  comptr->matches = cgdb_malloc (sizeof (char *) * (num_matches + 1));
   for (i = 0; i <= num_matches; ++i)
-    comptr->matches[i] = xstrdup (matches[i]);
+    comptr->matches[i] = cgdb_strdup (matches[i]);
 
   comptr->num_matches = num_matches;
   comptr->max_length = max_length;
@@ -662,7 +668,7 @@ version_info (void)
 {
   static char buf[MAXLINE];
   sprintf (buf, "%s %s\r\n%s", "CGDB", VERSION,
-	   "Copyright 2002-2006 Bob Rossi and Mike Mueller.\n"
+	   "Copyright 2002-2007 Bob Rossi and Mike Mueller.\n"
 	   "CGDB is free software, covered by the GNU General Public License, and you are\n"
 	   "welcome to change it and/or distribute copies of it under certain conditions.\n"
 	   "There is absolutely no warranty for CGDB.\n");
@@ -717,7 +723,13 @@ parse_long_options (int *argc, char ***argv)
 	  exit (0);
 	case 'd':
 	  debugger_path = strdup (optarg);
-	  n += 2;
+	  if (optarg == (*argv)[n + 1]) {
+	    /* optarg is in next argv (-d foo) */
+	    n += 2;
+	  } else {
+	    /* optarg is in this argv (-dfoo) */
+	    n++;
+	  }
 	  break;
 	case 'h':
 	  usage ();
@@ -729,6 +741,11 @@ parse_long_options (int *argc, char ***argv)
 
   *argc -= n;
   *argv += n;
+
+  if (**argv && strcmp(**argv, "--") == 0) {
+    (*argc)--;
+    (*argv)++;
+  }
 }
 
 /* init_home_dir: Attempts to create a config directory in the user's home
@@ -815,6 +832,21 @@ user_input (void)
 {
   static int key, val;
 
+  /* Clear the current map sets. Give the KUI the map sets 
+   * that should be used with the current focus.
+   */
+  val = kui_manager_clear_map_sets (kui_ctx);
+  if (val == -1)
+  {
+    logger_write_pos (logger, __FILE__, __LINE__, "user_input error");
+    return -1;
+  }
+
+  if (if_get_focus () == CGDB)
+    val = kui_manager_add_map_set (kui_ctx, kui_map);
+  else if (if_get_focus () == GDB)
+    val = kui_manager_add_map_set (kui_ctx, kui_imap);
+
   key = kui_manager_getkey (kui_ctx);
   if (key == -1)
     {
@@ -839,12 +871,12 @@ user_input (void)
   /* Process the key */
   if (kui_term_is_cgdb_key (key))
     {
-      char *seqbuf = (char *) kui_manager_get_raw_data (kui_ctx);
-
+      char *seqbuf = kui_term_get_ascii_char_sequence_from_key (key);
       if (seqbuf == NULL)
 	{
 	  logger_write_pos (logger, __FILE__, __LINE__,
-			    "kui_manager_get_raw_data error");
+			    "kui_term_get_ascii_char_sequence_from_key error %d",
+			    key);
 	  return -1;
 	}
       else
@@ -860,6 +892,25 @@ user_input (void)
   return 0;
 }
 
+/**
+ * This will process all the input that the KUI has.
+ *
+ * \return
+ * 0 on success or -1 on error
+ */
+static int
+user_input_loop ()
+{
+  do {
+    if (user_input () == -1) {
+      logger_write_pos (logger, __FILE__, __LINE__, "user_input_loop failed");
+      return -1;
+    }
+  } while (kui_manager_cangetkey (kui_ctx));
+
+  return 0;
+}
+
 static void
 process_commands (struct tgdb *tgdb)
 {
@@ -1328,13 +1379,18 @@ main_loop (void)
 	  return -1;
 
       /* Input received:  Handle it */
-      if (FD_ISSET (STDIN_FILENO, &rset))
-	if (user_input () == -1)
-	  {
-	    logger_write_pos (logger, __FILE__, __LINE__,
-			      "user_input failed");
-	    return -1;
-	  }
+      if (FD_ISSET (STDIN_FILENO, &rset)) {
+        int val = user_input_loop ();
+        /* The below condition happens on cygwin when user types ctrl-z
+         * select returns (when it shouldn't) with the value of 1. the
+         * user input loop gets called, the kui gets called and does a
+         * non blocking read which returns EAGAIN. The kui then passes
+         * the -1 up the stack with out making any more system calls. */
+        if (val == -1 && errno == EAGAIN)
+          continue;
+        else if (val == -1)
+          return -1;
+      }
 
       /* child's ouptut -> stdout
        * The continue is important I think. It allows all of the child
@@ -1487,6 +1543,65 @@ create_and_init_pair ()
   return 0;
 }
 
+int
+update_kui (cgdbrc_config_option_ptr option)
+{
+  kui_manager_set_terminal_escape_sequence_timeout (kui_ctx, cgdbrc_get_key_code_timeoutlen ());
+  kui_manager_set_key_mapping_timeout (kui_ctx, cgdbrc_get_mapped_key_timeoutlen ());
+
+  return 0;
+}
+
+int
+init_kui (void)
+{
+  kui_ctx = kui_manager_create (STDIN_FILENO, cgdbrc_get_key_code_timeoutlen (),
+					      cgdbrc_get_mapped_key_timeoutlen ());
+  if (!kui_ctx)
+    {
+      logger_write_pos (logger, __FILE__, __LINE__,
+			"Unable to initialize input library");
+      cleanup ();
+      exit (-1);
+    }
+
+  kui_map = kui_ms_create ();
+  if (!kui_map)
+    {
+      logger_write_pos (logger, __FILE__, __LINE__,
+			"Unable to initialize input library");
+      cleanup ();
+      exit (-1);
+    }
+
+  kui_imap = kui_ms_create ();
+  if (!kui_imap)
+    {
+      logger_write_pos (logger, __FILE__, __LINE__,
+			"Unable to initialize input library");
+      cleanup ();
+      exit (-1);
+    }
+
+  if (kui_manager_add_map_set (kui_ctx, kui_map) == -1)
+    {
+      logger_write_pos (logger, __FILE__, __LINE__,
+			"Unable to initialize input library");
+      cleanup ();
+      exit (-1);
+    }
+
+  /* Combine the cgdbrc config package with libkui. If any of the options
+   * below change, update the KUI.  Currently, the handles are not kept around,
+   * because CGDB never plans on detaching. */
+  cgdbrc_attach (CGDBRC_TIMEOUT, &update_kui, NULL);
+  cgdbrc_attach (CGDBRC_TIMEOUT_LEN, &update_kui, NULL);
+  cgdbrc_attach (CGDBRC_TTIMEOUT, &update_kui, NULL);
+  cgdbrc_attach (CGDBRC_TTIMEOUT_LEN, &update_kui, NULL);
+
+  return 0;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1539,11 +1654,9 @@ main (int argc, char *argv[])
       exit (-1);
     }
 
-  kui_ctx = kui_manager_create (STDIN_FILENO);
-  if (!kui_ctx)
+  if (init_kui () == -1)
     {
-      logger_write_pos (logger, __FILE__, __LINE__,
-			"Unable to initialize input library");
+      logger_write_pos (logger, __FILE__, __LINE__, "init_kui error");
       cleanup ();
       exit (-1);
     }
@@ -1600,9 +1713,3 @@ main (int argc, char *argv[])
   cleanup ();
   return 0;
 }
-
-int
-cgdb_set_esc_sequence_timeout (int msec)
-{
-  return kui_manager_set_terminal_escape_sequence_timeout (kui_ctx, msec);
-}
