@@ -58,9 +58,6 @@
 #include "highlight_groups.h"
 
 int sources_syntax_on = 1;
-int auto_source_reload = 0;
-enum ArrowStyle config_arrowstyle = ARROWSTYLE_SHORT;
-extern int highlight_tabstop;
 
 /* --------------- */
 /* Local Functions */
@@ -269,9 +266,9 @@ static int load_file(struct list_node *node)
         int i;
         node->buf.length = node->orig_buf.length;
         node->buf.max_width = node->orig_buf.max_width;
-        node->buf.tlines = xmalloc(sizeof(char *) * node->orig_buf.length);
+        node->buf.tlines = cgdb_malloc(sizeof(char *) * node->orig_buf.length);
         for ( i = 0; i < node->orig_buf.length; i++ )
-            node->buf.tlines[i] = xstrdup ( node->orig_buf.tlines[i] );
+            node->buf.tlines[i] = cgdb_strdup ( node->orig_buf.tlines[i] );
     }
 
     /* Allocate the breakpoints array */
@@ -303,6 +300,8 @@ static void draw_current_line(struct sviewer *sview, int line, int lwidth) {
     int column_offset = 0;              /* Text to skip due to arrow */
     int arrow_attr;
     int highlight_attr;
+    enum ArrowStyle config_arrowstyle = cgdbrc_get (CGDBRC_ARROWSTYLE)->variant.arrow_style;
+    int highlight_tabstop = cgdbrc_get (CGDBRC_TABSTOP)->variant.int_val;
 
     if (hl_groups_get_attr (hl_groups_instance, HLG_ARROW, &arrow_attr) == -1)
       return;
@@ -554,7 +553,10 @@ int source_display(struct sviewer *sview, int focus)
     int lwidth;
     int line;
     int i;
-		int attr = 0;
+    int attr = 0, sellineno;
+
+    if (hl_groups_get_attr (hl_groups_instance, HLG_SELECTED_LINE_NUMBER, &sellineno) == -1)
+      return -1;
 
     /* Check that a file is loaded */
     if (sview->cur == NULL || sview->cur->buf.tlines == NULL){
@@ -675,12 +677,12 @@ int source_display(struct sviewer *sview, int focus)
             /* Ordinary lines */
             else{
                 if ( focus && sview->cur->sel_line == line )
-                    wattron(sview->win, A_BOLD);
+                    wattron(sview->win, sellineno);
                     
                 wprintw(sview->win, fmt, line+1);
                 
                 if ( focus && sview->cur->sel_line == line )
-                    wattroff(sview->win, A_BOLD);
+                    wattroff(sview->win, sellineno);
 
                 if (focus)
                     wattron(sview->win, A_BOLD);
@@ -898,6 +900,7 @@ int source_reload ( struct sviewer *sview, const char *path, int force ) {
     time_t timestamp;
     struct list_node *cur;
     struct list_node *prev = NULL;
+    int auto_source_reload = cgdbrc_get (CGDBRC_AUTOSOURCERELOAD)->variant.int_val;
 
     if ( !path )
         return -1;
