@@ -19,20 +19,6 @@ extern "C"
 {
 #endif
 
-/**
- * This should be depricated and removed from TGDB.
- */
-#ifndef TRUE
-#define TRUE  1
-#endif
-
-/**
- * This should be depricated and removed from TGDB.
- */
-#ifndef FALSE
-#define FALSE 0
-#endif
-
 #include "tgdb_list.h"
 
  /* A reference to a command that has been created by TGDB */
@@ -46,7 +32,7 @@ extern "C"
  */
 /******************************************************************************/
 
-//@{
+/*@{*/
 
  /**
   * This will print a client generated command to stderr.
@@ -83,7 +69,7 @@ extern "C"
   void tgdb_types_append_command (struct tgdb_list *command_list,
 				  struct tgdb_response *response);
 
-//@}
+/*@}*/
 
  /**
   * The client can give any of these commands to TGDB through 
@@ -249,13 +235,16 @@ extern "C"
         const char *command;
       } console_command;
 
-      struct {
-      } info_sources;
+      /* info_sources; */
 
       struct {
 	const char *file;
       } filename_pair;
 
+      struct {
+	int on_startup;
+      } current_location;
+
       struct {
 	/** This is the command that libtgdb should run through the debugger */
 	enum tgdb_command_type c;
@@ -383,8 +372,7 @@ extern "C"
       } update_source_files;
 
       /* header == TGDB_SOURCES_DENIED */
-      struct {
-      } sources_denied;
+      /* sources_denied; */
       
       /* header == TGDB_FILENAME_PAIR */
       struct {
